<?php
namespace controllers;

class device extends \console {

    private $str_dns_path;
    private $str_sys_conf_path;

    private $str_vpn_dev = false;
    private $str_vpn_local_vip = false;
    private $str_vpn_remote_vip = false;

    private $str_country_code = false;

    private function  init () {
        $this->str_dns_path = \application::env("DNS_CONFIG_PATH");
        $this->str_sys_conf_path = \application::env("SYS_CONFIG_PATH");
    }

    function watch_dog(){
        $this->init();
        $str_ready_file = "/dev/shm/router.ready";

        if(!file_exists($str_ready_file)){
            return;
        }

        $str_lock_file = "/dev/shm/watchdog.lock";
        if(file_exists($str_lock_file)){
            $int_lock_time = (int) file_get_contents($str_lock_file);
            if($this->timestamp-$int_lock_time>=300){
                unlink($str_lock_file);
            }else{
                return;
            }
        }
        file_put_contents($str_lock_file,$this->timestamp);

        do{
            if(!$this->is_wan_ready()){
                unlink($str_ready_file);
                $this->log->info("WAN Fail , Restarting...");
                $this->do_stop();
                sleep(30);
                $this->do_start();
            }else{
                unlink($str_lock_file);
                return;
            }

            sleep(30);

            if($this->is_wan_ready()){
                unlink($str_lock_file);
                $this->log->info("WAN Restart success.");
                return;
            }else{
                file_put_contents($str_lock_file,$this->timestamp);
                sleep(30);
            }

        } while(true);




    }


    function do_start () {
        $this->init();

        $this->output->line("init system env...");

        $this->do_cmd("mkdir /dev/shm/log;chmod a+w /dev/shm/log -R");
        $this->do_cmd("iptables -t nat -A POSTROUTING -j MASQUERADE");
        $this->do_cmd('echo 22158 > /proc/sys/fs/file-max');

        $this->do_cmd("ip route flush table 10");    //内部和优先流量
        $this->do_cmd("ip route flush table 100");   //去往本地端口的流量（可指定IP），境内使用设定
        $this->do_cmd("ip route flush table 200");   //去往VPN端口的流量（可指定IP），境外使用设定
        $this->do_cmd("ip route flush table 300");   //去往本地端口的流量（保底默认）

        if(file_exists($this->str_sys_conf_path . DIRECTORY_SEPARATOR . ".new")){
            $this->new_router_init();
        }

        $this->output->line("init lan...");
        $this->init_lan();

        $this->output->line("init wan...");
        $this->init_wan();

        $this->output->line("init squid...");
        $this->init_squid();

        //$this->output->line($str_cmd);


    }

    private function new_router_init(){
        $this->output->line("init for new router...");

        $obj_os = new \os();
        $arr_interfaces = $obj_os->get_interfaces_info();
        if(count($arr_interfaces)<3){
            $this->output->fatal("Interface must up to 3");
            die;
        }
        $str_WAN = false;
        $str_LAN_PORTS = false;
        foreach ($arr_interfaces as $str_interface => $arr_contents){
            if($str_interface!="lo"){
                if(!$str_WAN){
                    $str_WAN=$str_interface;
                }else{
                    $str_LAN_PORTS .= $str_interface . ",";
                }
            }
        }
        $str_LAN_PORTS=substr($str_LAN_PORTS,0,strlen($str_LAN_PORTS)-1);

        $str_setting_file = $this->str_sys_conf_path . DIRECTORY_SEPARATOR . "router.conf";
        $str_setting = file_get_contents($str_setting_file);
        $str_setting = str_replace("{WAN}",$str_WAN,$str_setting);
        $str_setting = str_replace("{LAN_PORTS}",$str_LAN_PORTS,$str_setting);
        file_put_contents($str_setting_file,$str_setting);

        unlink($this->str_sys_conf_path . DIRECTORY_SEPARATOR . ".new");

    }

    private function init_wan () {
        $str_wan = $this->get_setting("WAN");
        $this->do_cmd("ifconfig $str_wan down");

        $str_wan_type = $this->get_setting("WAN_TYPE");
        switch ($str_wan_type) {
            case "static":
                $this->init_wan_static();
                break;
            case "dhcp":
                $this->init_wan_dhcp();
                break;
            case "pppoe":
                $this->init_wan_pppoe();
                break;

        }

        $str_ctl_port = $this->get_setting("CTL_PORT");
        $str_mon_port = $str_ctl_port + 10003;

        #$this->do_cmd("autossh -M {$str_mon_port} -NfR  0.0.0.0:{$str_ctl_port}:127.0.0.1:22 tunnel@tunnel.zhufeng.me &");

    }

    private function init_wan_pppoe(){
        $str_pppoe_username = $this->get_setting("PPPOE_USERNAME");
        $str_pppoe_password = $this->get_setting("PPPOE_PASSWORD");
        $str_wan = $this->get_setting("WAN");

        $str_pppoe_conf = file_get_contents($this->str_sys_conf_path . DIRECTORY_SEPARATOR . "pppoe.tpl");
        $str_pppoe_conf = str_replace("{WAN}",$str_wan,$str_pppoe_conf);
        $str_pppoe_conf = str_replace("{PPPOE_USERNAME}",$str_pppoe_username,$str_pppoe_conf);

        file_put_contents("/etc/ppp/peers/pppoe",$str_pppoe_conf);

        $str_pppoe_chap_secret = file_get_contents($this->str_sys_conf_path . DIRECTORY_SEPARATOR . "chap-secrets.tpl");
        $str_pppoe_chap_secret = str_replace("{PPPOE_USERNAME}",$str_pppoe_username,$str_pppoe_chap_secret);
        $str_pppoe_chap_secret = str_replace("{PPPOE_PASSWORD}",$str_pppoe_password,$str_pppoe_chap_secret);

        file_put_contents("/etc/ppp/chap-secrets",$str_pppoe_chap_secret);

        $str_pppoe_pap_secret = file_get_contents($this->str_sys_conf_path . DIRECTORY_SEPARATOR . "pap-secrets.tpl");
        $str_pppoe_pap_secret = str_replace("{PPPOE_USERNAME}",$str_pppoe_username,$str_pppoe_pap_secret);
        $str_pppoe_pap_secret = str_replace("{PPPOE_PASSWORD}",$str_pppoe_password,$str_pppoe_pap_secret);

        file_put_contents("/etc/ppp/pap-secrets",$str_pppoe_pap_secret);



        $this->do_cmd("poff -a");
        $this->do_cmd("ifconfig {$str_wan} down");

        $this->do_cmd("ifconfig {$str_wan} up");
        $this->do_cmd("pon pppoe");

        $int_retry = 0;
        while (!$this->is_wan_ready() && $int_retry < 3) {
            sleep(10);
            $int_retry++;
        }



    }


    private function init_wan_static () {
        $str_wan = $this->get_setting("WAN");
        $str_ip = $this->get_setting("WAN_IP");
        $str_mask = $this->get_setting("WAN_MASK");
        $str_gw = $this->get_setting("WAN_GATEWAY");
        $str_local_dns = $this->get_setting("LOCAL_DNS");

        $str_cmd = "ifconfig $str_wan $str_ip netmask $str_mask up";
        $this->do_cmd($str_cmd);
        $str_cmd = "ip route add 0/0 via $str_gw dev $str_wan";
        $this->do_cmd($str_cmd);
        $str_cmd = "echo nameserver {$str_local_dns} >> /etc/resolv.conf";
        $this->do_cmd($str_cmd);

        $int_retry = 0;
        while (!$this->is_wan_ready() && $int_retry < 3) {
            sleep(10);
            $int_retry++;
        }

        $this->wan_on_up();

        return;
    }

    private function init_wan_dhcp () {
        $str_wan = $this->get_setting("WAN");
        $str_cmd = "dhclient $str_wan";
        $this->do_cmd($str_cmd);

        $int_retry = 0;
        while (!$this->is_wan_ready() && $int_retry < 3) {
            sleep(10);
            $int_retry++;
        }

        $this->wan_on_up();

        return;
    }

    private function init_lan(){
        $str_lan = $this->get_setting("LAN");
        $str_cmd = "brctl addbr $str_lan";
        $this->do_cmd($str_cmd);
        $str_cmd = "brctl stp $str_lan on";
        $this->do_cmd($str_cmd);

        $str_lan_ports = $this->get_setting("LAN_PORTS");
        $arr_lan_ports = explode(",", $str_lan_ports);
        foreach ($arr_lan_ports as $str_lan_port) {
            $str_cmd = "brctl addif $str_lan $str_lan_port";
            $this->do_cmd($str_cmd);
            $str_cmd = "ifconfig $str_lan_port 0.0.0.0 up";
            $this->do_cmd($str_cmd);
        }

        $str_lan_ip = $this->get_setting("LAN_IP");
        $str_lan_mask = $this->get_setting("LAN_MASK");

        $str_cmd = "ifconfig $str_lan $str_lan_ip netmask $str_lan_mask up";
        $this->do_cmd($str_cmd);

    }

    private function init_vpn(){
        $this->init();

        $is_run = $this->do_cmd("/bin/pidof openvpn");
        if($is_run){
            $this->output->line("VPN already running..");
            return ;
        }
        
        if($this->get_setting("VPN_TYPE")=="P2P"){
            $str_cmd = "/usr/sbin/openvpn --config {$this->str_sys_conf_path}/openvpn-p2p.conf --remote ";
            $str_cmd .= $this->get_setting("P2P_SERVER") . " --daemon ";
            $str_cmd .= "--secret  " . $this->get_setting("P2P_KEY_FILE") . " ";
            $str_cmd .= "--ifconfig " . $this->get_setting("P2P_LOCAL_VIP") . " " . $this->get_setting("P2P_REMOTE_VIP") . " ";
            $str_cmd .= "--up  \"" . KKF_ROOT_PATH . "/twd events:vpn_on_up\" ";
            $str_cmd .= "--down \"" . KKF_ROOT_PATH . "/twd events:vpn_on_down\" ";
        }else{
            $str_cmd = "/usr/sbin/openvpn --config {$this->str_sys_conf_path}/openvpn-net.conf --remote ";
            $str_cmd .= $this->get_setting("TW_TUNNEL") . " --daemon ";
            $str_cmd .= "--ca {$this->str_sys_conf_path}/keys/ca.crt ";
            $str_cmd .= "--key {$this->str_sys_conf_path}/keys/client.key ";
            $str_cmd .= "--cert {$this->str_sys_conf_path}/keys/client.crt ";
            $str_cmd .= "--up  \"" . KKF_ROOT_PATH . "/twd events:vpn_on_up\" ";
            $str_cmd .= "--down \"" . KKF_ROOT_PATH . "/twd events:vpn_on_down\" ";
        }
        $this->do_cmd($str_cmd);
    }


    function vpn_on_up () {
        $this->init();

        $str_vpn_dev = $this->argv[2];
        $str_vpn_remote_vip = $this->argv[6];
        $str_vpn_local_vip = $this->argv[5];

        $this->str_vpn_dev = $str_vpn_dev;
        $this->str_vpn_local_vip = $str_vpn_local_vip;
        $this->str_vpn_remote_vip = $str_vpn_remote_vip;

        if($this->get_setting("VPN_TYPE")=="P2P"){
            
            $this->do_cmd("ip route add $str_vpn_local_vip/32 dev $str_vpn_dev table 10");
            $this->do_cmd("ip route add $str_vpn_remote_vip/32 dev $str_vpn_dev table 10");
            //$this->do_cmd("ip route add 0/0 dev $str_vpn_dev table 200");
            //$this->do_cmd("ip route add {$this->get_setting('P2P_REMOTE_NET')} dev $str_vpn_dev table 200");


        }else{
            
            $this->do_cmd("ip route add $str_vpn_remote_vip/32 dev $str_vpn_dev table 10");
            $this->do_cmd("ip route add $str_vpn_local_vip/32 via $str_vpn_remote_vip dev $str_vpn_dev table 10");
            //$this->do_cmd("ip route add 0/0 via $str_vpn_remote_vip dev $str_vpn_dev table 200");

        }

        $str_dns_server = $this->get_setting("TW_DNS");
        $this->do_cmd("ip route add $str_dns_server/32 dev $str_vpn_dev table 10");

        switch($this->get_setting("ROUTE_MODE")){
            case 'SMART':
                $this->set_vpn_route_smart();
                break;
            case 'GLOBAL':
                $this->set_vpn_route_global();
                break;
            case 'INTERNAL':
                $this->set_vpn_route_internal();
                break;
            default: 
                $this->set_vpn_route_smart();
        }

        $this->init_dhcp();

    }

    function set_vpn_route_global(){
        if($this->get_setting("VPN_TYPE")=="P2P"){
            $this->do_cmd("ip route add 0/0 dev $this->str_vpn_dev table 200");
        }else{
            $this->do_cmd("ip route add 0/0 via  $this->str_vpn_remote_vip dev $this->str_vpn_dev table 200");
        }
    }
    function set_vpn_route_internal(){

        $str_vpn_remote_nets = $this->get_setting("VPN_REMOTE_NETS");
        $arr_vpn_remote_nets = explode(",",$str_vpn_remote_nets);
        foreach ($arr_vpn_remote_nets as $ip){
            if($ip){
                if($this->get_setting("VPN_TYPE")=="P2P"){
                    $this->do_cmd("ip route add $ip dev $this->str_vpn_dev table 200");
                }else{
                    $this->do_cmd("ip route add $ip via $this->str_vpn_remote_vip dev $this->str_vpn_dev table 200");    
                }
            }
        }

        
    }

    function set_vpn_route_smart(){

        if(!$this->str_country_code){
            $this->str_country_code = $this->get_country_code();
        }

        
        $str_local_connection_data = file_get_contents("/dev/shm/local_connection_data");
        $obj_local_connection_data = json_decode($str_local_connection_data);

        $str_local_gw = $obj_local_connection_data->str_local_gw ;
        $str_local_dev = $obj_local_connection_data->str_local_dev ;

        $str_wan_type = $this->get_setting("WAN_TYPE");

        $str_cn_ips = file_get_contents("/var/trueword/update/cn_ip_list.txt");
        $arr_cn_ips = explode("\n", $str_cn_ips);

        if($this->str_country_code=="CN"){
            foreach ($arr_cn_ips as $ip) {
                if($ip) {
                    if($str_wan_type=="pppoe"){
                        $cmd = "ip route add $ip dev $str_local_dev table 100";
                    }else {
                        $cmd = "ip route add $ip via $str_local_gw dev $str_local_dev table 100";
                    }
                    //$this->output->line($cmd);
                    $this->do_cmd($cmd);
                    
                }
            }
            if($this->get_setting("VPN_TYPE")=="P2P"){
                $this->do_cmd("ip route add 0/0 dev $this->str_vpn_dev table 200");
            }else{
                $this->do_cmd("ip route add 0/0 via $this->str_vpn_remote_vip dev $this->str_vpn_dev table 200");    
            }
        }else{
            foreach ($arr_cn_ips as $ip) {
                if($ip) {
                    if($this->get_setting("VPN_TYPE")=="P2P"){
                        $cmd="ip route add $ip dev $this->str_vpn_dev table 200";
                    }else{
                        $cmd="ip route add $ip via $this->str_vpn_remote_vip dev $this->str_vpn_dev table 200";
                    }
                    //$this->output->line($cmd);
                    $this->do_cmd($cmd);
                }
            }
        }
        $this->set_vpn_route_internal();
        
    }

    function vpn_on_down () {

        //$this->do_cmd("ip rule del from all lookup 10 pref 10");
        $this->do_cmd("ip rule del from all lookup 100 pref 100");
        $this->do_cmd("ip rule del from all lookup 200 pref 200");
        //$this->do_cmd("ip rule del from all lookup 300 pref 300");
    }



    private function init_dhcp () {

        $pid = $this->do_cmd("pidof dnsmasq");
        if($pid){
            $this->do_cmd("kill $pid");
        }

        $str_lan = $this->get_setting("LAN");
        $str_lan_ip = $this->get_setting("LAN_IP");
        $str_lan_mask = $this->get_setting("LAN_MASK");

        $str_cmd = "/usr/sbin/dnsmasq -u dnsmasq -7 {$this->str_dns_path} -C {$this->str_sys_conf_path}/dnsmasq.conf ";
        $str_cmd .= "--interface=\"{$str_lan},lo\" ";
        $str_cmd .= "--listen-address=\"{$str_lan_ip},127.0.0.1\" ";

        $str_dhcp_start = $this->get_setting("DHCP_START");
        $str_dhcp_expire = $this->get_setting("DHCP_EXPIRE");

        $int_dhcp_start = ip2long($str_dhcp_start);
        $int_dhcp_end = $int_dhcp_start + $this->get_setting("DHCP_NUMBER");
        $str_dhcp_end = long2ip($int_dhcp_end);

        $str_cmd .= "--dhcp-range=\"{$str_dhcp_start},{$str_dhcp_end},{$str_lan_mask},{$str_dhcp_expire}\" ";
        $str_cmd .= "--dhcp-option=\"3,{$str_lan_ip}\" ";
        $str_cmd .= "--dhcp-option=\"6,{$str_lan_ip}\" ";

        $this->do_cmd($str_cmd);

        $str_lan_cidr = $this->mask2cidr($str_lan_mask);
        $str_lan_network = $this->get_network_ip($str_lan_ip, $str_lan_mask);

        $str_cmd = "ip route add {$str_lan_network}/{$str_lan_cidr} dev $str_lan table 10";
        $this->do_cmd($str_cmd);

        file_put_contents("/dev/shm/router.ready",$this->timestamp);

    }


    function init_squid () {

        if(!$this->get_setting("PROXY_ENABLED")){
            return;
        }

        $str_squid_conf = file_get_contents($this->str_sys_conf_path . DIRECTORY_SEPARATOR . "squid.conf.tpl");

        $str_lan_ip = $this->get_setting("LAN_IP");
        $str_lan_mask = $this->get_setting("LAN_MASK");
        $str_lan_cidr = $this->mask2cidr($str_lan_mask);
        $str_lan_network = $this->get_network_ip($str_lan_ip, $str_lan_mask);
        $str_lan_network = $str_lan_network . "/" . $str_lan_cidr;

        $str_squid_conf = str_replace("{LAN_NETWORK}", $str_lan_network, $str_squid_conf);

        $str_proxy_port = $this->get_setting("PROXY_PORT");
        $str_squid_conf = str_replace("{PROXY_PORT}", $str_proxy_port, $str_squid_conf);

        file_put_contents($this->str_sys_conf_path . DIRECTORY_SEPARATOR . "squid.conf", $str_squid_conf);

        $this->do_cmd("/usr/sbin/squid3 -z -f {$this->str_sys_conf_path}/squid.conf");
        $this->do_cmd("/usr/sbin/squid3 -YC -f {$this->str_sys_conf_path}/squid.conf");

        $this->do_cmd("iptables -t nat -A PREROUTING -p tcp -m tcp --dport 80 -j REDIRECT --to-ports {$str_proxy_port}");

    }

    function wan_on_down(){
        $this->init();
    }

    function wan_on_up(){

        $this->init();

        $this->do_heartbeat(false);

        $this->output->line("setting DNS ...");

        $this->set_dns();

        $obj_os = new \os();

        $this->output->line("updating route files...");
        
	    $str_update_file = $this->str_sys_conf_path . DIRECTORY_SEPARATOR . "lastupdate";
        if(!file_exists($str_update_file)) {
            $this->output->line("Lastupdate file not exist ... ");
            $this->do_update();
        } else {
            $int_lastupdate = file_get_contents($str_update_file);
            if($this->timestamp - $int_lastupdate > 86400) {
                $this->output->line("Lastupdate file expired ... ");
                $this->do_update();
                file_put_contents($str_update_file,$this->timestamp);
            }
        }

        $str_wan_type = $this->get_setting("WAN_TYPE");

        $vpn_server_ip =  "169.254.0.1";
        if($this->get_setting("VPN_TYPE")=="P2P"){
            $vpn_server_ip = $this->get_setting("P2P_SERVER");
            $vpn_server_ip = explode(" ",$vpn_server_ip);
            $vpn_server_ip = $vpn_server_ip[0];
        }else{
            $vpn_server_ip = $this->get_setting("TW_TUNNEL");
            $vpn_server_ip = explode(" ",$vpn_server_ip);
            $vpn_server_ip = $vpn_server_ip[0];
        }
        if($vpn_server_ip){
            $vpn_server_ip = gethostbyname($vpn_server_ip);
        }

        $hb_server_ip = "169.254.0.1";
        $hb_url = $this->get_setting("HEARTBEAT_URL");
        $arr_parse = parse_url($hb_url);
        if($arr_parse){
            $hb_server_ip = gethostbyname($arr_parse['host']);
        }

        if($str_wan_type=="pppoe"){
            $str_local_dev = $this->argv[2];
            $str_p2p = $this->get_uplink_info($str_local_dev)['dest'];
            $this->do_cmd("ip route add $str_p2p/32 dev $str_local_dev table 10");
            $this->do_cmd("iptables -t mangle -D POSTROUTING -o $str_local_dev -p tcp -m tcp --tcp-flags SYN,RST SYN -j TCPMSS --clamp-mss-to-pmtu");
            $this->do_cmd("iptables -t mangle -I POSTROUTING -o $str_local_dev -p tcp -m tcp --tcp-flags SYN,RST SYN -j TCPMSS --clamp-mss-to-pmtu");
            $this->do_cmd("ip route add $vpn_server_ip/32 dev $str_local_dev table 10");
            $this->do_cmd("ip route add $hb_server_ip/32 dev $str_local_dev table 10");
            $this->do_cmd("ip route add 0/0 dev $str_local_dev table 300");
        }else {
            $str_local_dev = $this->get_setting("WAN");
            $str_local_gw = $obj_os->get_gateway_by_interface($str_local_dev);
            $str_uplink_network = $this->get_uplink_info($str_local_dev)['network'];
            $this->do_cmd("ip route add $str_uplink_network dev $str_local_dev table 10");
            $this->do_cmd("ip route add $vpn_server_ip via $str_local_gw dev $str_local_dev table 10");
            $this->do_cmd("ip route add $hb_server_ip via $str_local_gw dev $str_local_dev table 10");
            $this->do_cmd("ip route add 0/0 via $str_local_gw dev $str_local_dev table 300");
        }

        $obj_local_connection_data = json_decode('{}');
        $obj_local_connection_data->str_local_gw = $str_local_gw;
        $obj_local_connection_data->str_local_dev = $str_local_dev;

        $str_local_connection_data = json_encode($obj_local_connection_data);
        file_put_contents("/dev/shm/local_connection_data",$str_local_connection_data);


        $this->do_cmd("ip rule add from all lookup 10 pref 10");
        $this->do_cmd("ip rule add from all lookup 100 pref 100");
        $this->do_cmd("ip rule add from all lookup 200 pref 200");
        $this->do_cmd("ip rule add from all lookup 300 pref 300");




        if($this->get_setting("ROUTE_MODE")=="DISABLED"){
            $this->init_dhcp();

        }else{
            $this->output->line("connecting to access network...");
            $this->init_vpn();
        }
        return;
        

    }

    function do_restart(){
        $this->output->line("Restartting router ..");
        $this->do_stop();
        $this->output->line("Watting for 10 sec ..");
        sleep(10);
        $this->do_start();
        return;
    }


    function do_stop () {

        $this->init();

        $str_ready_file = "/dev/shm/router.ready";
        $this->output->line("Remove state file ..");
        $this->do_cmd("rm -f $str_ready_file");

        $this->output->line("Stopping LAN bridge ..");
        $str_lan = $this->get_setting("LAN");
        $this->do_cmd("ifconfig $str_lan down");

        $this->output->line("Stopping LAN ports ..");
        $str_lan_ports = $this->get_setting("LAN_PORTS");

        $str_wan_type = $this->get_setting("WAN_TYPE");

        if($str_wan_type=="pppoe"){
            $this->do_cmd("poff -a");
        }

        $str_wan = $this->get_setting("WAN");
        $this->do_cmd("ifconfig $str_wan down");


        $arr_lan_ports = explode(",", $str_lan_ports);

        foreach ($arr_lan_ports as $str_lan_port) {
            $str_cmd = "ifconfig $str_lan_port down";
            $this->do_cmd($str_cmd);
            $str_cmd = "brctl delif $str_lan $str_lan_port";
            $this->do_cmd($str_cmd);
        }

        $this->output->line("Stopping dnsmasq ..");
        $pid = $this->do_cmd("pidof dnsmasq");
        $this->do_cmd("kill $pid");
        $this->output->line("Stopping squid3 ..");
        $pid = $this->do_cmd("pidof squid3");
        $this->do_cmd("kill $pid");
        $this->do_cmd("killall squid3");

        $this->do_cmd("brctl delbr $str_lan");

        $this->output->line("Stopping openvpn ..");
        $pid = $this->do_cmd("pidof openvpn");
        $this->do_cmd("kill $pid");

        $this->do_cmd("ip rule del from all lookup 10 pref 10");
        $this->do_cmd("ip rule del from all lookup 100 pref 100");
        $this->do_cmd("ip rule del from all lookup 200 pref 200");
        $this->do_cmd("ip rule del from all lookup 300 pref 300");

        $this->do_cmd("ip route flush table 10");
        $this->do_cmd("ip route flush table 100");
        $this->do_cmd("ip route flush table 200");
        $this->do_cmd("ip route flush table 300");

    }

    function set_dns(){

        $this->do_cmd("rm " . $this->str_dns_path . DIRECTORY_SEPARATOR . "*.conf -f");

        $str_wan_type = $this->get_setting("WAN_TYPE");

        if($str_wan_type=="static"){
            $str_dns_server = $this->get_setting("LOCAL_DNS");
        }else{
            $str_resolv_file = file_get_contents("/etc/resolv.conf");
            $str_dns_server = \helper::get_str_by_tag($str_resolv_file,"nameserver ","\n");
        }

        if($str_dns_server){
            $str_server = "server=" . $str_dns_server . "\n";
        }else{
            $str_server = "server=114.114.114.114 \n";
        }

        file_put_contents($this->str_dns_path . DIRECTORY_SEPARATOR . "server.conf", $str_server);

        if($this->get_setting("ROUTE_MODE")=="SMART"){
            $str_country_code = $this->get_country_code();
            $this->str_country_code = $str_country_code;

            $this->output->line("Country Code : " . $str_country_code);
            $str_domains_file = false;
            if($str_country_code=="CN"){
                $str_domains_file = "/var/trueword/update/cn_block_urls.txt";
            }else{
                $str_domains_file = "/var/trueword/update/china_domains.txt";
            }

            $str_domains = file_get_contents($str_domains_file);
            $arr_domains = explode("\n", $str_domains);
            $str_conf = null;
            foreach ($arr_domains as $row) {
                if($row) {
                    $str_conf .= "server=/$row/" . $this->get_setting("TW_DNS") . "\n";
                }
            }
            file_put_contents($this->str_dns_path . DIRECTORY_SEPARATOR . "specific_domains.conf", $str_conf);
            
        }

        if($this->get_setting("ROUTE_MODE")!="DISABLED"){
            $str_dns = "server=/twd/10.10.10.10\n";
            file_put_contents($this->str_dns_path . DIRECTORY_SEPARATOR . "internal_domains.conf", $str_dns);
        }

        return;
    }

    function do_update(){
        $this->init();

        $this->log->info("Saving last fingerprint...");
        $str_fingerprint = false;
        if(file_exists("/var/trueword/update/fingerprint")){
            $str_fingerprint = file_get_contents("/var/trueword/update/fingerprint");
        }

        $this->log->info("Startting update...");
        $str_update_file = $this->str_sys_conf_path . DIRECTORY_SEPARATOR . "lastupdate";
        $str_tmp_file_path = "/tmp/tw_update";
        $this->do_cmd("rm $str_tmp_file_path -rf");
        $str_update_url = $this->get_setting("UPDATE_URL");

        $this->do_cmd("mkdir $str_tmp_file_path");
        $str_http_code = \helper::http_download($str_update_url . "/MD5SUM", $str_tmp_file_path . "/MD5SUM");
        if($str_http_code!="200"){
            $this->log->info("Update data download failed.");
            return false;
        }
        
        $str_http_code = \helper::http_download($str_update_url . "/data.tar", $str_tmp_file_path . "/data.tar");
        if($str_http_code!="200"){
            $this->log->info("Update data download failed.");
            return false;
        }

        $str_verify = $this->do_cmd("cd $str_tmp_file_path;md5sum -c MD5SUM");
        if(strpos($str_verify,"FAILED")){
            $this->log->info("Update data check failed.");
            return false;
        }

        $this->do_cmd("rm /var/trueword/update -rf");
        $this->do_cmd("mkdir -p /var/trueword/update");

        $this->do_cmd("cd $str_tmp_file_path;tar xf data.tar");
        $this->do_cmd("cp $str_tmp_file_path/*.txt /var/trueword/update");
        $this->do_cmd("cat $str_tmp_file_path/*.txt | md5sum > /var/trueword/update/fingerprint");

        file_put_contents($str_update_file, $this->timestamp);
        $this->do_cmd("rm $str_tmp_file_path -rf");

        if(file_exists("/var/trueword/update/fingerprint")){
            $str_fingerprint_current = file_get_contents("/var/trueword/update/fingerprint");
            if($str_fingerprint != $str_fingerprint_current){
                $this->log->info("Startting reload router rule ...");
                $this->do_restart();
            }
        }

        return true;


    }

    public function do_heartbeat($need_delay=true){
        $this->init();
        if($need_delay){
            $delay = rand(1,299);
            $this->log->info("Heartbeat dalay {$delay} secs...");
            sleep($delay);
        }
        
        $this->log->info("Startting heartbeat...");
        $url = $this->get_setting("HEARTBEAT_URL");
        $device_id = $this->do_cmd("hostname");
        $device_id = str_replace("\n","",$device_id);
        $url .= "?device_id={$device_id}&device_name={$device_id}";
        $result = \helper::http_request($url);
        $this->log->info("Heartbeat is done.");
        return;

    }


    private function get_setting ($key) {
        $str_setting_file = $this->str_sys_conf_path . DIRECTORY_SEPARATOR . "router.conf";
        $arr_config = parse_ini_file($str_setting_file);
        if(array_key_exists($key, $arr_config)) {
            return $arr_config[$key];
        }
        return false;
    }

    private function do_cmd ($cmd) {
        $this->log->debug("exec: " . $cmd);
        $result = \application::system_exec($cmd);
        $this->log->debug("exec result: " . $result);
        return $result;

    }

    private function is_wan_ready () {
        $obj_os = new \os();
        $is = $obj_os->is_ping_ok("baidu.com");
        if($is) {
            return true;
        }
        $is = $obj_os->is_ping_ok("taobao.com");
        if($is) {
            return true;
        }
        $is = $obj_os->is_ping_ok("google.com");
        if($is) {
            return true;
        }
        return false;
    }


    private function get_network_ip ($str_ip, $str_mask) {
        $int_ip = ip2long($str_ip);
        $int_mask = ip2long($str_mask);
        $int_network = $int_ip & $int_mask;
        return long2ip($int_network);
    }

    private function mask2cidr ($str_mask) {
        $long = ip2long($str_mask);
        $base = ip2long('255.255.255.255');
        return 32 - log(($long ^ $base) + 1, 2);
    }

    private function get_uplink_info ($str_dev) {
        $obj_os = new \os();
        if(!$obj_os->is_interface_exist($str_dev)) {
            return false;
        }
        $arr_interfaces = $obj_os->get_interfaces_info();
        if(!array_key_exists($str_dev, $arr_interfaces)) {
            return false;
        }
        $arr_interface = $arr_interfaces[$str_dev];

        $str_ip = $arr_interface['ipv4'];
        $str_netmask = $arr_interface['netmask'];

        $result['network'] = $this->get_network_ip($str_ip, $str_netmask) . "/" . $this->mask2cidr($str_netmask);
        $result['gateway'] = $obj_os->get_gateway_by_interface($str_dev);

        if(array_key_exists("p2p",$arr_interface)){
            $result['p2p'] = $arr_interface['p2p'];
        }

        return $result;

    }

    private function get_country_code(){
        $this->init();        
        if($this->get_setting("FORCE_COUNTRY")!="AUTO"){
            if($this->get_setting("FORCE_COUNTRY")){
                return $this->get_setting("FORCE_COUNTRY");
            }
        }

        $str_data = false;
        $count = 0 ;
        while(!$str_data && $count <= 3 ){
            $count += 1;
            $str_data = file_get_contents("https://api.ip.la/en?json");
            if($count>0){
                sleep(5);
            }
        }

        if(!$str_data){
            $str_data = "CN";
        }
        
        $obj_data = json_decode($str_data);
        $str_country_code = $obj_data->location->country_code;
        return $str_country_code;
    }



}
