<?php

class console_table {


    private $arr_header = false;
    private $int_style = 0;
    private $no_ansi = false;
    private $str_border_color = "white";

    function __construct ($int_style = 0, $str_border_color = false) {

        $is_tput = \application::system_exec("which tput");
        if(!$is_tput) {
            throw new Exception("need tput command");
        }

        $this->int_style = $int_style;

        if(!$str_border_color) {
            $this->no_ansi = true;
        } else {
            $this->str_border_color = $str_border_color;
        }

        return;

    }

    function add_header_col ($str_text, $int_width, $str_index = false) {
        if(!$str_index) {
            $str_index = md5($str_text);
        }

        $int_current_width = (int)\application::system_exec("tput cols");
        if($this->int_style == 0) {
            $int_need_width = $this->get_header_width() + $int_width;
        } else {
            $int_need_width = $this->get_header_width() + $int_width + 1;
            $int_need_width += 2;
        }

        if($int_current_width < $int_need_width) {
            die("ERROR: Current screen too narrow \n");
        }

        $this->arr_header[$str_index]['text'] = $str_text;
        $this->arr_header[$str_index]['width'] = $int_width;

    }

    function print_line($arr_data,$str_space_full=" ",$is_last_line=false){
        $arr_line = $this->arr_header;

        $arr_data = array_pad($arr_data,count($arr_line),$str_space_full);

        foreach($arr_data as $key => $content){
            if(array_key_exists($key,$arr_line)){
                $arr_line[$key]['content']=$content;
                unset($arr_data[$key]);
            }
        }

        foreach($arr_data as $key => $content){
            foreach($arr_line as $line_key => $line){
                if(!array_key_exists("content",$line)){
                    $arr_line[$line_key]['content']=array_shift($arr_data);
                    break;
                }
            }

        }

        if($this->int_style == 1) {
            $this->out(self::coloring("|", $this->str_border_color, false, "default",$this->no_ansi));
        }

        foreach ($arr_line as $col) {
            $this->out(str_pad($col['content'], $col['width'], " "));
            if($this->int_style == 1) {
                $this->out(self::coloring("|", $this->str_border_color, false, "default",$this->no_ansi));
            }
        }

        if($this->int_style==1){
            $this->line();
            if($is_last_line){
                $this->print_top_and_footer_line();
            }else{
                $this->print_sectioning_line();
            }
        }

        $this->line();

    }

    function print_header () {
        if(!$this->arr_header) {
            die("add_header_col first \n");
        }

        if($this->int_style == 1) {
            $this->print_top_and_footer_line();
        }

        if($this->int_style == 1) {
            $this->out(self::coloring("|", $this->str_border_color, false, "default",$this->no_ansi));
        }

        foreach ($this->arr_header as $col) {
            $text = str_pad($col['text'], $col['width'], " ");
            $text = self::coloring($text,"white",false,"bold",$this->no_ansi);
            $this->out($text);
            if($this->int_style == 1) {
                $this->out(self::coloring("|", $this->str_border_color, false, "default",$this->no_ansi));
            }
        }


        $this->line();

        $this->print_sectioning_line();


    }

    private function print_top_and_footer_line () {
        $int_length = $this->get_header_width();
        if($this->int_style == 1) {
            $int_length += 1;
        }

        $this->line(self::coloring(str_pad("-", $int_length, "-"), $this->str_border_color, false, "default",$this->no_ansi));

    }

    private function print_sectioning_line () {

        $str_line = null;

        if($this->int_style == 1) {
            $str_line .= "|";

        }
        if($this->int_style == 1) {
            foreach ($this->arr_header as $col) {
                $str_line .= str_pad("+", $col['width'] + 1, "-", STR_PAD_LEFT);
            }
        } else {
            $str_line .= str_pad("-", $this->get_header_width(), "-");
        }

        if($this->int_style == 1) {
            $str_line = substr($str_line, 0, strlen($str_line) - 1);
            $str_line .= "|";
        }
        $this->out(self::coloring($str_line, $this->str_border_color, false, "default",$this->no_ansi));

        $this->line();

    }

    private function line ($str_text = null) {
        $this->out($str_text);
        $this->out("\n");
    }

    private function out ($str_text) {
        echo $str_text;
    }

    public static function coloring ($str_text, $str_color, $str_bg_color = false, $str_style = "default",$no_ansi = false) {
        $str_ansi = null;
        if($no_ansi) {
            return $str_text;
        }

        switch($str_style){
            case "bold";
                $str_ansi.="\x1b[1";
                break;
            case "underline":
                $str_ansi.="\x1b[4";
                break;
            case "flash":
                $str_ansi.="\x1b[5";
                break;
            case "rev":
                $str_ansi.="\x1b[7";
                break;
            default:
                $str_ansi.="\x1b[0";
                break;
        }


        switch ($str_color) {
            case "black":
                $str_ansi .= ";30";
                break;
            case "red":
                $str_ansi .= ";31";
                break;
            case "green":
                $str_ansi .= ";32";
                break;
            case "yellow":
                $str_ansi .= ";33";
                break;
            case "blue":
                $str_ansi .= ";34";
                break;
            case "kermes":
                $str_ansi .= ";35";
                break;
            case "cyan":
                $str_ansi .= ";36";
                break;
            default:
                $str_ansi .= ";37";
                break;
        }
        switch ($str_bg_color) {
            case "white":
                $str_ansi .= ";47m";
                break;
            case "red":
                $str_ansi .= ";41m";
                break;
            case "green":
                $str_ansi .= ";42m";
                break;
            case "yellow":
                $str_ansi .= ";43m";
                break;
            case "blue":
                $str_ansi .= ";44m";
                break;
            case "kermes":
                $str_ansi .= ";45m";
                break;
            case "cyan":
                $str_ansi .= ";46m";
                break;
            case "black":
                $str_ansi .= ";40m";
                break;

            default:
                $str_ansi .= "m";
                break;
        }

        $str_ansi = $str_ansi . $str_text . "\x1b[0m";

        return $str_ansi;


    }

    private function get_header_width () {
        if(!$this->arr_header) {
            return 0;
        }

        $int = 0;

        foreach ($this->arr_header as $col) {
            $int = $int + $col['width'];
            if($this->int_style == 1) {
                $int++;
            }

        }

        return $int;
    }

    function clear_header () {

    }


}