<?php

class kkconf {

    private $str_conf_path = null;
    private $str_conf_filename = null;
    private $arr_conf = false;

    function __construct ($str_conf_path, $str_conf_name) {
        if(!is_dir($str_conf_path)) {
            die("Config file path not found");
        }
        $this->str_conf_path = $str_conf_path;
        $this->str_conf_name = $str_conf_name;

        $this->str_conf_filename = $str_conf_path . DIRECTORY_SEPARATOR . $str_conf_name . ".conf";

        //if(!is_writeable($this->str_conf_filename)) {
        //die("Config file is readonly");
        //}

        if(file_exists($this->str_conf_filename)) {
            $str_conf_content = file_get_contents($this->str_conf_filename);
            $this->arr_conf = json_decode($str_conf_content, true);
        }

        return;

    }

    function clear () {
        $this->arr_conf = false;
        return;
    }

    function commit () {

        if(!$this->arr_conf) {
            return false;
        }
        $str_json = $this->get_json();
        $result = file_put_contents($this->str_conf_filename, $str_json);
        return $result;

    }

    function get_json () {
        if(!$this->arr_conf) {
            return false;
        }
        $str_json = json_encode($this->arr_conf, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
        return $str_json;
    }

    function get_section ($str_section_type, $str_section_name = false) {
        if(!$this->arr_conf){
            return false;

        }

        $arr_result = false;

        if(!$str_section_name) {
            $str_needle = $str_section_type;
            foreach ($this->arr_conf as $key => $value) {
                if(substr($key, 0, strlen($str_needle)) == $str_needle) {
                    $arr_result[$key] = $value;
                }
            }
        } else {
            $str_needle = $str_section_type . "|" . $str_section_name;
            foreach ($this->arr_conf as $key => $value) {
                if($key == $str_needle) {
                    $arr_result[$key] = $value;
                }
            }

        }





        return $arr_result;
    }

    function add_section ($str_section_type, $str_section_name = false) {

        if(!self::is_vaild_name($str_section_type)) {
            return false;
        }

        if($str_section_name && (!self::is_vaild_name($str_section_name))){
                return false;
        }

        if($str_section_name) {
            $str_section_type .= "|" . $str_section_name;
        }

        $this->arr_conf[$str_section_type] = array();

        return;
    }

    function del_section ($str_section_type, $str_section_name = false) {

        if(!self::is_vaild_name($str_section_type) || !self::is_vaild_name($str_section_name)) {
            return false;
        }
        if($str_section_name) {
            $str_section_type .= "|" . $str_section_name;
        }

        unset($this->arr_conf[$str_section_type]);

        return;
    }

    function get_option ($str_section_type, $str_section_name = false, $str_option_key) {
        $result = false;

        $arr_section = $this->get_section($str_section_type,$str_section_name);

        if(!$arr_section){
            return false;
        }


        foreach($arr_section as $key => $value) {


            if(array_key_exists($str_option_key, $value)) {
                $mix_option = $value[$str_option_key];
                if(is_array($mix_option)) {
                    foreach ($mix_option as $nkey => $nvalue) {
                        $result[] = $nvalue;
                    }
                } else {
                    $result[] = $mix_option;
                }
            }
        }
        if(is_array($result)){
            $result=array_values($result);
            if(count($result)==1){
                return $result[0];
            }else{
                return $result;
            }
        }

        return $result;
    }

    function add_option ($str_section_type, $str_section_name = false, $str_option_key, $str_option_value) {

        if($str_section_name) {
            $str_section_type .= "|" . $str_section_name;
        }

        if(!array_key_exists($str_section_type, $this->arr_conf)) {
            return false;
        }

        $arr_conf = $this->arr_conf[$str_section_type];
        if(!array_key_exists($str_option_key, $arr_conf)) {
            $this->arr_conf[$str_section_type][$str_option_key] = $str_option_value;
            return true;
        }

        $arr_option = array();
        if(is_array($this->arr_conf[$str_section_type][$str_option_key])) {
            $arr_option = $this->arr_conf[$str_section_type][$str_option_key];
        } else {
            $arr_option[] = $this->arr_conf[$str_section_type][$str_option_key];
            $arr_option[] = $str_option_value;
        }
        $this->arr_conf[$str_section_type][$str_option_key] = $arr_option;
        return true;

    }

    function del_option ($str_section_type, $str_section_name = false, $str_option_key, $str_option_value = false) {

        if($str_section_name) {
            $str_section_type .= "|" . $str_section_name;
        }

        if(!array_key_exists($str_section_type, $this->arr_conf)) {
            return false;
        }

        if(!is_array($this->arr_conf[$str_section_type][$str_option_key])) {
            unset($this->arr_conf[$str_section_type][$str_option_key]);
            return true;
        }

        $arr_conf = $this->arr_conf[$str_section_type][$str_option_key];
        $result = false;
        foreach ($arr_conf as $key => $value) {
            if($value == $str_option_value) {
                unset($arr_conf[$key]);
                $arr_conf = array_values($arr_conf);
                $result = true;
            }
        }

        if(count($arr_conf) == 1) {
            $arr_conf = $arr_conf[0];
        }

        $this->arr_conf[$str_section_type][$str_option_key] = $arr_conf;

        return $result;


    }

    private static function is_vaild_name ($str) {
        if(preg_match('/^[a-zA-Z][a-zA-Z0-9_-]{2,20}$/', $str)) {
            return true;
        } else {
            return false;
        }
    }

}