<?php

// for Linux X86

class os_core {

    public function flush_route(){
        $this->do_cmd("ip route flush table 50");
        $this->do_cmd("ip route flush table 80");
    }

    private function do_cmd ($str_cmd, $error = false) {
        //$this->log->info($str_cmd);
        $result = \application::system_exec($str_cmd, $error);
        return $result;
    }

    /**
     * 获取端口当前网络速率
     *
     * @param string $str_interface 采样端口
     * @param int $int_srate 采样周期（秒）
     * @param string $str_way 采样方向（all/out/in)
     *
     * @return int 端口速率，单位：bps  bit/s
     *         string 可读结果
     */
    public function get_speed_by_interface ($str_interface, $str_way = "all", $is_readable = false, $int_srate = 3) {
        if(!$this->is_interface_exist($str_interface)) {
            return false;
        }
        $str = $this->do_cmd("ifstat -i {$str_interface} {$int_srate} 1");
        $row = explode("\n", $str);
        if(array_key_exists(2, $row)) {
            $row = $row[2];
        } else {
            return false;
        }
        while (strstr($row, "  ")) {
            $row = str_replace("  ", " ", $row);
            $row = trim($row);
        }
        $row = explode(" ", $row);

        $int_speed_in = 0;
        $int_speed_out = 0;
        $int_speed_all = 0;

        if(array_key_exists(0, $row)) {
            $int_speed_in = (float)$row[0] * 1024 * 8;
        }

        if(array_key_exists(1, $row)) {
            $int_speed_out = (float)$row[1] * 1024 * 8;
        }

        if($int_speed_in > 0 && $int_speed_out > 0) {
            $int_speed_all = $int_speed_in + $int_speed_out;
        }

        $result = false;

        switch ($str_way) {
            case "in":
                $result = intval($int_speed_in);
                break;
            case "out":
                $result = intval($int_speed_out);
                break;
            case "all":
                $result = intval($int_speed_all);
                break;
        }

        if($is_readable) {
            return \helper::digi_to_readable($result, "bit")."/s";
        } else {
            return $result;
        }

    }


    public function get_gateway_by_interface ($str_interface) {
        $str = $this->do_cmd("ip route | grep 'dev {$str_interface}' ");
        if(!$str) {
            return false;
        }
        $result = \helper::get_str_by_tag($str, "via", "dev");
        if(\network::is_ip($result)) {
            return $result;
        }
        return false;
    }

    public function ping_test ($str_remote, $int_test_time = 1, $int_interval = 0.01, $str_outgoing_interface = false) {
        $str_cmd = "ping -q -i $int_interval -w $int_test_time ";
        if($str_outgoing_interface) {
            $str_cmd .= "-I $str_outgoing_interface ";
        }
        $str_cmd .= $str_remote;

        $str = $this->do_cmd($str_cmd);
        $loss = \helper::get_str_by_tag($str, "received,", "%");
        $loss = (float)$loss / 100;

        if($loss < 1) {
            $row = \helper::get_str_by_tag($str, "rtt", "ms");
        }
        $row = explode("=", $row);
        $row = explode("/", $row[1]);
        $rtt = (float)$row[1];

        if($loss>=1){
            $rtt = 65535;
        }

        $result['loss'] = $loss;
        $result['rtt'] = $rtt;

        return $result;
    }

    public function is_tunnel_exist ($str_local_ip, $str_remote_ip) {
        $str = $this->do_cmd("ip tunnel list remote {$str_remote_ip} local {$str_local_ip}");

        if(!$str) {
            return false;
        }
        return true;
    }

    public function is_xfrm_exist ($str_local_ip, $str_remote_ip) {
        $is_out = $this->do_cmd("ip xfrm state list src {$str_local_ip} dst {$str_remote_ip}");
        $is_in = $this->do_cmd("ip xfrm state list src {$str_remote_ip} dst {$str_local_ip}");
        return (bool)($is_out && $is_in);
    }

    private function get_spi ($str_local_ip, $str_remote_ip) {
        $result = false;
        $out_str = $this->do_cmd("ip xfrm state list src {$str_local_ip} dst {$str_remote_ip}");
        $in_str = $this->do_cmd("ip xfrm state list dst {$str_local_ip} src {$str_remote_ip}");
        if($out_str) {
            $result['spi_out'] = \helper::get_str_by_tag($out_str, 'spi ', " ");
            $result['spi_out'] = substr($result['spi_out'], 2, strlen($result['spi_out']) - 2);
        }
        if($in_str) {
            $result['spi_in'] = \helper::get_str_by_tag($in_str, 'spi ', " ");
            $result['spi_in'] = substr($result['spi_in'], 2, strlen($result['spi_in']) - 2);
        }
        return $result;
    }

    private function get_local_vip_by_tunnel ($str_tunnel_name, $ipv6 = false) {

        $str = $this->do_cmd("ip addr show $str_tunnel_name");
        if(!$str) {
            return false;
        }

        if($ipv6) {
            $result = \helper::get_str_by_tag($str, "inet6 ", " ");
        } else {
            $result = \helper::get_str_by_tag($str, "inet ", " ");
        }

        list($result) = explode("/", $result);

        return $result;

    }

    public function get_tunnels_info () {
        $str = $this->do_cmd("ip tunnel show");
        $arr = explode("\n", $str);

        $result = array();

        foreach ($arr as $row) {
            if($row) {
                $str_tunnel_name = substr($row, 0, strpos($row, ":"));
                $result[$str_tunnel_name]['remote_ip'] = \helper::get_str_by_tag($row, "remote ", " ");
                $result[$str_tunnel_name]['local_ip'] = \helper::get_str_by_tag($row, "local ", " ");
                $result[$str_tunnel_name]['local_vip'] = $this->get_local_vip_by_tunnel($str_tunnel_name);
                $arr_spi = $this->get_spi($result[$str_tunnel_name]['local_ip'], $result[$str_tunnel_name]['remote_ip']);
                if($arr_spi) {
                    $result[$str_tunnel_name]['spi_in'] = $arr_spi['spi_in'];
                    $result[$str_tunnel_name]['spi_out'] = $arr_spi['spi_out'];
                }
                if($result[$str_tunnel_name]['remote_ip'] == "any") {
                    unset($result[$str_tunnel_name]);
                }
            }
        }
        return $result;
    }


    public function is_ping_ok ($str_target_ip, $str_outbound_interface = false) {

        if($str_outbound_interface) {
            $str_ping = $this->do_cmd("export LANG=en;ping -I $str_outbound_interface -w 3 $str_target_ip");
        } else {
            $str_ping = $this->do_cmd("export LANG=en;ping -w 3 $str_target_ip");
        }

        $int_package_ok = (int)\helper::get_str_by_tag($str_ping, "packets transmitted,", "received");
        if($int_package_ok > 0) {
            return true;
        }

        return false;
    }

    public function is_ip_exist ($str_ip) {
        $arr_ips = $this->get_interfaces_info();
        foreach ($arr_ips as $str_if => $arr_ip) {
            if(array_search($str_ip, $arr_ip)) {
                return $str_if;
            }
        }
        return false;

    }

    public function get_interfaces_info () {
        $result = array();
        $str = $this->do_cmd("export LANG=en;/sbin/ifconfig -a");
        $arr_ifs = explode("\n\n", $str);
        foreach ($arr_ifs as $str_if) {
            $arr_if_infos = explode("\n", $str_if);
            foreach ($arr_if_infos as $str_if_info) {
                $str_if_info= $this->trim_space($str_if_info);
                if(strstr($str_if_info, "flags")) {
                    $arr = explode(":", $str_if_info);
                    $result[$arr[0]] = array();
                    $str_current_if = $arr[0];
                }
                if(strstr($str_if_info, "txqueuelen")) {
                    $str = \helper::get_str_by_tag($str_if_info, "(", ")");
                    $result[$str_current_if]['encap'] = trim($str);
                }
                if(strstr($str_if_info, "inet ")) {
                    $arr= explode(" ",$str_if_info);
                    $result[$str_current_if]['ipv4'] = $arr[1];
                    $result[$str_current_if]['netmask'] = $arr[3];
                }

                if(strstr($str_if_info, "RX packets")) {
                    $arr= explode(" ",$str_if_info);
                    $result[$str_current_if]['RX'] = $arr[4];
                }
                if(strstr($str_if_info, "TX packets")) {
                    $arr= explode(" ",$str_if_info);
                    $result[$str_current_if]['TX'] = $arr[4];
                }
                if(strstr($str_if_info, "inet6 ")) {
                    $arr= explode(" ",$str_if_info);
                    $result[$str_current_if]['ipv6'] = $arr[1];
                }
                if(strstr($str_if_info, "destination")) {
                    $str = \helper::get_str_by_tag($str_if_info, "destination","\n");
                    $result[$str_current_if]['dest'] = $str;
                }
            }
        }
        return $result;
    }

    private function trim_space($str){
        while (strstr($str,"  ")){
            $str= str_replace("  "," ",$str);
        }
        return trim($str);
    }

    public function get_local_dns(){
        $str_resolv = file_get_contents("/etc/resolv.conf");
        $str_dns = \helper::get_str_by_tag($str_resolv,"nameserver","\n");
        return $str_dns;
    }

    public function is_interface_exist ($str_interface) {
        $arr_test = $this->get_interfaces_info();
        return array_key_exists($str_interface, $arr_test);

    }
}